@@ -39,9 +39,10 @@ from jenkins_jobs.errors import MissingAttributeError
 from jenkins_jobs.errors import InvalidAttributeError
 import jenkins_jobs.modules.base
 from jenkins_jobs.modules.helpers import copyartifact_build_selector
+from jenkins_jobs.modules.helpers import convert_mapping_to_xml
 
 
-def base_param(parser, xml_parent, data, do_default, ptype):
+def base_param(registry, xml_parent, data, do_default, ptype):
     pdef = XML.SubElement(xml_parent, ptype)
     XML.SubElement(pdef, 'name').text = data['name']
     XML.SubElement(pdef, 'description').text = data.get('description', '')
@@ -54,7 +55,7 @@ def base_param(parser, xml_parent, data, do_default, ptype):
     return pdef
 
 
-def string_param(parser, xml_parent, data):
+def string_param(registry, xml_parent, data):
     """yaml: string
     A string parameter.
 
@@ -70,11 +71,11 @@ def string_param(parser, xml_parent, data):
             default: bar
             description: "A parameter named FOO, defaults to 'bar'."
     """
-    base_param(parser, xml_parent, data, True,
+    base_param(registry, xml_parent, data, True,
                'hudson.model.StringParameterDefinition')
 
 
-def promoted_param(parser, xml_parent, data):
+def promoted_param(registry, xml_parent, data):
     """yaml: promoted build
     A promoted build parameter.
     Requires the Jenkins :jenkins-wiki:`Promoted Builds Plugin
@@ -92,7 +93,7 @@ def promoted_param(parser, xml_parent, data):
        :language: yaml
 
     """
-    pdef = base_param(parser, xml_parent, data, False,
+    pdef = base_param(registry, xml_parent, data, False,
                       'hudson.plugins.promoted__builds.parameters.'
                       'PromotedBuildParameterDefinition')
     try:
@@ -104,7 +105,7 @@ def promoted_param(parser, xml_parent, data):
         'promotion-name', None)
 
 
-def password_param(parser, xml_parent, data):
+def password_param(registry, xml_parent, data):
     """yaml: password
     A password parameter.
 
@@ -120,11 +121,11 @@ def password_param(parser, xml_parent, data):
             default: 1HSC0Ts6E161FysGf+e1xasgsHkgleLh09JUTYnipPvw=
             description: "A parameter named FOO."
     """
-    base_param(parser, xml_parent, data, True,
+    base_param(registry, xml_parent, data, True,
                'hudson.model.PasswordParameterDefinition')
 
 
-def bool_param(parser, xml_parent, data):
+def bool_param(registry, xml_parent, data):
     """yaml: bool
     A boolean parameter.
 
@@ -141,11 +142,11 @@ def bool_param(parser, xml_parent, data):
             description: "A parameter named FOO, defaults to 'false'."
     """
     data['default'] = str(data.get('default', False)).lower()
-    base_param(parser, xml_parent, data, True,
+    base_param(registry, xml_parent, data, True,
                'hudson.model.BooleanParameterDefinition')
 
 
-def file_param(parser, xml_parent, data):
+def file_param(registry, xml_parent, data):
     """yaml: file
     A file parameter.
 
@@ -159,11 +160,11 @@ def file_param(parser, xml_parent, data):
             name: test.txt
             description: "Upload test.txt."
     """
-    base_param(parser, xml_parent, data, False,
+    base_param(registry, xml_parent, data, False,
                'hudson.model.FileParameterDefinition')
 
 
-def text_param(parser, xml_parent, data):
+def text_param(registry, xml_parent, data):
     """yaml: text
     A text parameter.
 
@@ -179,11 +180,11 @@ def text_param(parser, xml_parent, data):
             default: bar
             description: "A parameter named FOO, defaults to 'bar'."
     """
-    base_param(parser, xml_parent, data, True,
+    base_param(registry, xml_parent, data, True,
                'hudson.model.TextParameterDefinition')
 
 
-def label_param(parser, xml_parent, data):
+def label_param(registry, xml_parent, data):
     """yaml: label
     A node label parameter.
 
@@ -199,12 +200,12 @@ def label_param(parser, xml_parent, data):
             default: precise
             description: "The node on which to run the job"
     """
-    base_param(parser, xml_parent, data, True,
+    base_param(registry, xml_parent, data, True,
                'org.jvnet.jenkins.plugins.nodelabelparameter.'
                'LabelParameterDefinition')
 
 
-def node_param(parser, xml_parent, data):
+def node_param(registry, xml_parent, data):
     """yaml: node
     Defines a list of nodes where this job could potentially be executed on.
     Restrict where this project can be run, If your using a node or label
@@ -232,7 +233,7 @@ def node_param(parser, xml_parent, data):
        :language: yaml
 
     """
-    pdef = base_param(parser, xml_parent, data, False,
+    pdef = base_param(registry, xml_parent, data, False,
                       'org.jvnet.jenkins.plugins.nodelabelparameter.'
                       'NodeParameterDefinition')
     default = XML.SubElement(pdef, 'defaultSlaves')
@@ -258,12 +259,12 @@ def node_param(parser, xml_parent, data):
         data.get('allowed-multiselect', False)).lower()
 
 
-def choice_param(parser, xml_parent, data):
+def choice_param(registry, xml_parent, data):
     """yaml: choice
     A single selection parameter.
 
     :arg str name: the name of the parameter
-    :arg list choices: the available choices
+    :arg list choices: the available choices, first one is the default one.
     :arg str description: a description of the parameter (optional)
 
     Example::
@@ -276,7 +277,7 @@ def choice_param(parser, xml_parent, data):
               - glance
             description: "On which project to run?"
     """
-    pdef = base_param(parser, xml_parent, data, False,
+    pdef = base_param(registry, xml_parent, data, False,
                       'hudson.model.ChoiceParameterDefinition')
     choices = XML.SubElement(pdef, 'choices',
                              {'class': 'java.util.Arrays$ArrayList'})
@@ -285,7 +286,62 @@ def choice_param(parser, xml_parent, data):
         XML.SubElement(a, 'string').text = choice
 
 
-def run_param(parser, xml_parent, data):
+def credentials_param(registry, xml_parent, data):
+    """yaml: credentials
+    A credentials selection parameter. Requires the Jenkins
+    :jenkins-wiki:`Credentials Plugin
+    <Credentials+Plugin>`.
+
+    :arg str name: the name of the parameter
+    :arg str type: credential type (optional, default 'any')
+
+        :Allowed Values: * **any** Any credential type (default)
+                    * **usernamepassword** Username with password
+                    * **sshkey** SSH Username with private key
+                    * **secretfile** Secret file
+                    * **secrettext** Secret text
+                    * **certificate** Certificate
+
+    :arg bool required: whether this parameter is required (optional, default
+        false)
+    :arg string default: default credentials ID (optional)
+    :arg str description: a description of the parameter (optional)
+
+    Example::
+
+        .. literalinclude::
+            /../../tests/parameters/fixtures/credentials-param001.yaml
+           :language: yaml
+
+    """
+    cred_impl_types = {
+        'any': 'com.cloudbees.plugins.credentials.common.StandardCredentials',
+        'usernamepassword': 'com.cloudbees.plugins.credentials.impl.' +
+                            'UsernamePasswordCredentialsImpl',
+        'sshkey': 'com.cloudbees.jenkins.plugins.sshcredentials.impl.' +
+                  'BasicSSHUserPrivateKey',
+        'secretfile': 'org.jenkinsci.plugins.plaincredentials.impl.' +
+                      'FileCredentialsImpl',
+        'secrettext': 'org.jenkinsci.plugins.plaincredentials.impl.' +
+                      'StringCredentialsImpl',
+        'certificate': 'com.cloudbees.plugins.credentials.impl.' +
+                       'CertificateCredentialsImpl'
+    }
+
+    cred_type = data.get('type', 'any').lower()
+    if cred_type not in cred_impl_types:
+        raise InvalidAttributeError('type', cred_type, cred_impl_types.keys())
+
+    pdef = base_param(registry, xml_parent, data, False,
+                      'com.cloudbees.plugins.credentials.' +
+                      'CredentialsParameterDefinition')
+    XML.SubElement(pdef, 'defaultValue').text = data.get('default', '')
+    XML.SubElement(pdef, 'credentialType').text = cred_impl_types[cred_type]
+    XML.SubElement(pdef, 'required').text = str(data.get('required',
+                                                         False)).lower()
+
+
+def run_param(registry, xml_parent, data):
     """yaml: run
     A run parameter.
 
@@ -299,12 +355,13 @@ def run_param(parser, xml_parent, data):
        :language: yaml
 
     """
-    pdef = base_param(parser, xml_parent, data, False,
+    pdef = base_param(registry, xml_parent, data, False,
                       'hudson.model.RunParameterDefinition')
-    XML.SubElement(pdef, 'projectName').text = data['project-name']
+    mapping = [('project-name', 'projectName', None)]
+    convert_mapping_to_xml(pdef, data, mapping, fail_required=True)
 
 
-def extended_choice_param(parser, xml_parent, data):
+def extended_choice_param(registry, xml_parent, data):
     """yaml: extended-choice
     Creates an extended choice parameter where values can be read from a file
     Requires the Jenkins :jenkins-wiki:`Extended Choice Parameter Plugin
@@ -337,31 +394,29 @@ def extended_choice_param(parser, xml_parent, data):
     :arg str description-property-key: key for the value description
         property file (optional, default '')
     :arg str multi-select-delimiter: value between selections when the
-        parameter is a multi-select (optiona, default ',')
+        parameter is a multi-select (optional, default ',')
+    :arg str groovy-script: the groovy script contents (optional, default ',')
+    :arg str classpath: the classpath for the groovy script
+        (optional, default ',')
 
-    Example:
 
-    .. literalinclude:: \
-    /../../tests/parameters/fixtures/extended-choice-param001.yaml
-       :language: yaml
+    Minimal Example:
 
+        .. literalinclude:: \
+        /../../tests/parameters/fixtures/extended-choice-param-minimal.yaml
+           :language: yaml
+
+    Full Example:
+
+        .. literalinclude:: \
+        /../../tests/parameters/fixtures/extended-choice-param-full.yaml
+           :language: yaml
     """
-    pdef = base_param(parser, xml_parent, data, False,
+    pdef = base_param(registry, xml_parent, data, False,
                       'com.cwctravel.hudson.plugins.'
                       'extended__choice__parameter.'
                       'ExtendedChoiceParameterDefinition')
-    XML.SubElement(pdef, 'value').text = data.get('value', '')
-    XML.SubElement(pdef, 'visibleItemCount').text = str(data.get(
-        'visible-items', data.get('visible-item-count', 5)))
-    XML.SubElement(pdef, 'multiSelectDelimiter').text = data.get(
-        'multi-select-delimiter', ',')
-    XML.SubElement(pdef, 'quoteValue').text = str(data.get('quote-value',
-                                                  False)).lower()
-    XML.SubElement(pdef, 'defaultValue').text = data.get(
-        'default-value', '')
-    XML.SubElement(pdef, 'descriptionPropertyValue').text = data.get(
-        'value-description', '')
-    choice = data.get('type', 'single-select')
+
     choicedict = {'single-select': 'PT_SINGLE_SELECT',
                   'multi-select': 'PT_MULTI_SELECT',
                   'radio': 'PT_RADIO',
@@ -372,26 +427,27 @@ def extended_choice_param(parser, xml_parent, data):
                   'PT_RADIO': 'PT_RADIO',
                   'PT_CHECKBOX': 'PT_CHECKBOX',
                   'PT_TEXTBOX': 'PT_TEXTBOX'}
-
-    if choice in choicedict:
-        XML.SubElement(pdef, 'type').text = choicedict[choice]
-    else:
-        raise JenkinsJobsException("Type entered is not valid, must be one "
-                                   "of: single-select, multi-select, radio, "
-                                   "textbox or checkbox")
-    XML.SubElement(pdef, 'propertyFile').text = data.get('property-file', '')
-    XML.SubElement(pdef, 'propertyKey').text = data.get('property-key', '')
-    XML.SubElement(pdef, 'defaultPropertyFile').text = data.get(
-        'default-property-file', '')
-    XML.SubElement(pdef, 'defaultPropertyKey').text = data.get(
-        'default-property-key', '')
-    XML.SubElement(pdef, 'descriptionPropertyFile').text = data.get(
-        'description-property-file', '')
-    XML.SubElement(pdef, 'descriptionPropertyKey').text = data.get(
-        'description-property-key', '')
+    mapping = [
+        ('value', 'value', ''),
+        ('visible-items', 'visibleItemCount', 5),
+        ('multi-select-delimiter', 'multiSelectDelimiter', ','),
+        ('quote-value', 'quoteValue', False),
+        ('default-value', 'defaultValue', ''),
+        ('value-description', 'descriptionPropertyValue', ''),
+        ('type', 'type', 'single-select', choicedict),
+        ('property-file', 'propertyFile', ''),
+        ('property-key', 'propertyKey', ''),
+        ('default-property-file', 'defaultPropertyFile', ''),
+        ('default-property-key', 'defaultPropertyKey', ''),
+        ('description-property-file', 'descriptionPropertyFile', ''),
+        ('description-property-key', 'descriptionPropertyKey', ''),
+        ('groovy-script', 'groovyScript', ''),
+        ('classpath', 'groovyClasspath', ''),
+    ]
+    convert_mapping_to_xml(pdef, data, mapping, fail_required=True)
 
 
-def validating_string_param(parser, xml_parent, data):
+def validating_string_param(registry, xml_parent, data):
     """yaml: validating-string
     A validating string parameter
     Requires the Jenkins :jenkins-wiki:`Validating String Plugin
@@ -413,24 +469,33 @@ def validating_string_param(parser, xml_parent, data):
             regex: [A-Za-z]*
             msg: Your entered value failed validation
     """
-    pdef = base_param(parser, xml_parent, data, True,
+    pdef = base_param(registry, xml_parent, data, True,
                       'hudson.plugins.validating__string__parameter.'
                       'ValidatingStringParameterDefinition')
-    XML.SubElement(pdef, 'regex').text = data['regex']
-    XML.SubElement(pdef, 'failedValidationMessage').text = data['msg']
+    mapping = [
+        ('regex', 'regex', None),
+        ('msg', 'failedValidationMessage', None),
+    ]
+    convert_mapping_to_xml(pdef, data, mapping, fail_required=True)
 
 
-def svn_tags_param(parser, xml_parent, data):
+def svn_tags_param(registry, xml_parent, data):
     """yaml: svn-tags
     A svn tag parameter
     Requires the Jenkins :jenkins-wiki:`Parameterized Trigger Plugin
     <Parameterized+Trigger+Plugin>`.
 
     :arg str name: the name of the parameter
-    :arg str default: the default value of the parameter (optional)
-    :arg str description: a description of the parameter (optional)
     :arg str url: the url to list tags from
-    :arg str filter: the regular expression to filter tags
+    :arg str credentials-id: Credentials ID to use for authentication
+        (default '')
+    :arg str filter: the regular expression to filter tags (default '')
+    :arg str default: the default value of the parameter (default '')
+    :arg str description: a description of the parameter (default '')
+    :arg int max-tags: the number of tags to display (default '100')
+    :arg bool sort-newest-first: sort tags from newest to oldest (default true)
+    :arg bool sort-z-to-a: sort tags in reverse alphabetical order
+        (default false)
 
     Example::
 
@@ -442,18 +507,22 @@ def svn_tags_param(parser, xml_parent, data):
             url: http://svn.example.com/repo
             filter: [A-za-z0-9]*
     """
-    pdef = base_param(parser, xml_parent, data, True,
+    pdef = base_param(registry, xml_parent, data, True,
                       'hudson.scm.listtagsparameter.'
                       'ListSubversionTagsParameterDefinition')
-    XML.SubElement(pdef, 'tagsDir').text = data['url']
-    XML.SubElement(pdef, 'tagsFilter').text = data.get('filter', None)
-    XML.SubElement(pdef, 'reverseByDate').text = "true"
-    XML.SubElement(pdef, 'reverseByName').text = "false"
-    XML.SubElement(pdef, 'maxTags').text = "100"
-    XML.SubElement(pdef, 'uuid').text = "1-1-1-1-1"
+    mapping = [
+        ('url', 'tagsDir', None),
+        ('credentials-id', 'credentialsId', ''),
+        ('filter', 'tagsFilter', ''),
+        ('max-tags', 'maxTags', '100'),
+        ('sort-newest-first', 'reverseByDate', True),
+        ('sort-z-to-a', 'reverseByName', False),
+        ('', 'uuid', "1-1-1-1-1"),
+    ]
+    convert_mapping_to_xml(pdef, data, mapping, fail_required=True)
 
 
-def dynamic_choice_param(parser, xml_parent, data):
+def dynamic_choice_param(registry, xml_parent, data):
     """yaml: dynamic-choice
     Dynamic Choice Parameter
     Requires the Jenkins :jenkins-wiki:`Jenkins Dynamic Parameter Plug-in
@@ -478,10 +547,11 @@ def dynamic_choice_param(parser, xml_parent, data):
             remote: false
             read-only: false
     """
-    dynamic_param_common(parser, xml_parent, data, 'ChoiceParameterDefinition')
+    dynamic_param_common(registry, xml_parent, data,
+                         'ChoiceParameterDefinition')
 
 
-def dynamic_string_param(parser, xml_parent, data):
+def dynamic_string_param(registry, xml_parent, data):
     """yaml: dynamic-string
     Dynamic Parameter
     Requires the Jenkins :jenkins-wiki:`Jenkins Dynamic Parameter Plug-in
@@ -506,10 +576,11 @@ def dynamic_string_param(parser, xml_parent, data):
             remote: false
             read-only: false
     """
-    dynamic_param_common(parser, xml_parent, data, 'StringParameterDefinition')
+    dynamic_param_common(registry, xml_parent, data,
+                         'StringParameterDefinition')
 
 
-def dynamic_choice_scriptler_param(parser, xml_parent, data):
+def dynamic_choice_scriptler_param(registry, xml_parent, data):
     """yaml: dynamic-choice-scriptler
     Dynamic Choice Parameter (Scriptler)
     Requires the Jenkins :jenkins-wiki:`Jenkins Dynamic Parameter Plug-in
@@ -542,11 +613,11 @@ def dynamic_choice_scriptler_param(parser, xml_parent, data):
             remote: false
             read-only: false
     """
-    dynamic_scriptler_param_common(parser, xml_parent, data,
+    dynamic_scriptler_param_common(registry, xml_parent, data,
                                    'ScriptlerChoiceParameterDefinition')
 
 
-def dynamic_string_scriptler_param(parser, xml_parent, data):
+def dynamic_string_scriptler_param(registry, xml_parent, data):
     """yaml: dynamic-string-scriptler
     Dynamic Parameter (Scriptler)
     Requires the Jenkins :jenkins-wiki:`Jenkins Dynamic Parameter Plug-in
@@ -579,12 +650,12 @@ def dynamic_string_scriptler_param(parser, xml_parent, data):
             remote: false
             read-only: false
     """
-    dynamic_scriptler_param_common(parser, xml_parent, data,
+    dynamic_scriptler_param_common(registry, xml_parent, data,
                                    'ScriptlerStringParameterDefinition')
 
 
-def dynamic_param_common(parser, xml_parent, data, ptype):
-    pdef = base_param(parser, xml_parent, data, False,
+def dynamic_param_common(registry, xml_parent, data, ptype):
+    pdef = base_param(registry, xml_parent, data, False,
                       'com.seitenbau.jenkins.plugins.dynamicparameter.'
                       + ptype)
     XML.SubElement(pdef, '__remote').text = str(
@@ -604,30 +675,34 @@ def dynamic_param_common(parser, xml_parent, data, ptype):
         data.get('read-only', False)).lower()
 
 
-def dynamic_scriptler_param_common(parser, xml_parent, data, ptype):
-    pdef = base_param(parser, xml_parent, data, False,
+def dynamic_scriptler_param_common(registry, xml_parent, data, ptype):
+    pdef = base_param(registry, xml_parent, data, False,
                       'com.seitenbau.jenkins.plugins.dynamicparameter.'
                       'scriptler.' + ptype)
-    XML.SubElement(pdef, '__remote').text = str(
-        data.get('remote', False)).lower()
-    XML.SubElement(pdef, '__scriptlerScriptId').text = data.get(
-        'script-id', None)
     parametersXML = XML.SubElement(pdef, '__parameters')
     parameters = data.get('parameters', [])
     if parameters:
+        mapping = [
+            ('name', 'name', None),
+            ('value', 'value', None),
+        ]
         for parameter in parameters:
             parameterXML = XML.SubElement(parametersXML,
                                           'com.seitenbau.jenkins.plugins.'
                                           'dynamicparameter.scriptler.'
                                           'ScriptlerParameterDefinition_'
                                           '-ScriptParameter')
-            XML.SubElement(parameterXML, 'name').text = parameter['name']
-            XML.SubElement(parameterXML, 'value').text = parameter['value']
-    XML.SubElement(pdef, 'readonlyInputField').text = str(data.get(
-        'read-only', False)).lower()
+            convert_mapping_to_xml(
+                parameterXML, parameter, mapping, fail_required=True)
+    mapping = [
+        ('script-id', '__scriptlerScriptId', None),
+        ('remote', '__remote', False),
+        ('read-only', 'readonlyInputField', False),
+    ]
+    convert_mapping_to_xml(pdef, data, mapping, fail_required=True)
 
 
-def matrix_combinations_param(parser, xml_parent, data):
+def matrix_combinations_param(registry, xml_parent, data):
     """yaml: matrix-combinations
     Matrix combinations parameter
     Requires the Jenkins :jenkins-wiki:`Matrix Combinations Plugin
@@ -648,20 +723,17 @@ def matrix_combinations_param(parser, xml_parent, data):
     element_name = 'hudson.plugins.matrix__configuration__parameter.' \
                    'MatrixCombinationsParameterDefinition'
     pdef = XML.SubElement(xml_parent, element_name)
-    if 'name' not in data:
-        raise JenkinsJobsException('matrix-combinations must have a name '
-                                   'parameter.')
-    XML.SubElement(pdef, 'name').text = data['name']
-    XML.SubElement(pdef, 'description').text = data.get('description', '')
-    combination_filter = data.get('filter')
-    if combination_filter:
-        XML.SubElement(pdef, 'defaultCombinationFilter').text = \
-            combination_filter
+
+    mapping = [
+        ('name', 'name', None),
+        ('description', 'description', ''),
+        ('filter', 'defaultCombinationFilter', '')]
+    convert_mapping_to_xml(pdef, data, mapping, fail_required=True)
 
     return pdef
 
 
-def copyartifact_build_selector_param(parser, xml_parent, data):
+def copyartifact_build_selector_param(registry, xml_parent, data):
     """yaml: copyartifact-build-selector
 
     Control via a build parameter, which build the copyartifact plugin should
@@ -685,18 +757,16 @@ def copyartifact_build_selector_param(parser, xml_parent, data):
 
     t = XML.SubElement(xml_parent, 'hudson.plugins.copyartifact.'
                        'BuildSelectorParameter')
-    try:
-        name = data['name']
-    except KeyError:
-        raise MissingAttributeError('name')
-
-    XML.SubElement(t, 'name').text = name
-    XML.SubElement(t, 'description').text = data.get('description', '')
+    mapping = [
+        ('name', 'name', None),
+        ('description', 'description', ''),
+    ]
+    convert_mapping_to_xml(t, data, mapping, fail_required=True)
 
     copyartifact_build_selector(t, data, 'defaultSelector')
 
 
-def maven_metadata_param(parser, xml_parent, data):
+def maven_metadata_param(registry, xml_parent, data):
     """yaml: maven-metadata
     This parameter allows the resolution of maven artifact versions
     by contacting the repository and reading the maven-metadata.xml.
@@ -734,17 +804,18 @@ def maven_metadata_param(parser, xml_parent, data):
        :language: yaml
 
     """
-    pdef = base_param(parser, xml_parent, data, False,
+    pdef = base_param(registry, xml_parent, data, False,
                       'eu.markov.jenkins.plugin.mvnmeta.'
                       'MavenMetadataParameterDefinition')
-    XML.SubElement(pdef, 'repoBaseUrl').text = data.get('repository-base-url',
-                                                        '')
-    XML.SubElement(pdef, 'groupId').text = data.get('artifact-group-id', '')
-    XML.SubElement(pdef, 'artifactId').text = data.get('artifact-id', '')
-    XML.SubElement(pdef, 'packaging').text = data.get('packaging', '')
-    XML.SubElement(pdef, 'defaultValue').text = data.get('default-value', '')
-    XML.SubElement(pdef, 'versionFilter').text = data.get('versions-filter',
-                                                          '')
+    mapping = [
+        ('repository-base-url', 'repoBaseUrl', ''),
+        ('artifact-group-id', 'groupId', ''),
+        ('artifact-id', 'artifactId', ''),
+        ('packaging', 'packaging', ''),
+        ('default-value', 'defaultValue', ''),
+        ('versions-filter', 'versionFilter', ''),
+    ]
+    convert_mapping_to_xml(pdef, data, mapping, fail_required=True)
 
     sort_order = data.get('sorting-order', 'descending').lower()
     sort_dict = {'descending': 'DESC',
@@ -754,10 +825,171 @@ def maven_metadata_param(parser, xml_parent, data):
         raise InvalidAttributeError(sort_order, sort_order, sort_dict.keys())
 
     XML.SubElement(pdef, 'sortOrder').text = sort_dict[sort_order]
-    XML.SubElement(pdef, 'maxVersions').text = str(data.get(
-        'maximum-versions-to-display', 10))
-    XML.SubElement(pdef, 'username').text = data.get('repository-username', '')
-    XML.SubElement(pdef, 'password').text = data.get('repository-password', '')
+    mapping = [
+        ('maximum-versions-to-display', 'maxVersions', 10),
+        ('repository-username', 'username', ''),
+        ('repository-password', 'password', ''),
+    ]
+    convert_mapping_to_xml(pdef, data, mapping, fail_required=True)
+
+
+def hidden_param(parser, xml_parent, data):
+    """yaml: hidden
+    Allows you to use parameters hidden from the build with parameter page.
+    Requires the Jenkins :jenkins-wiki:`Hidden Parameter Plugin
+    <Hidden+Parameter+Plugin>`.
+
+    :arg str name: the name of the parameter
+    :arg str default: the default value of the parameter (optional)
+    :arg str description: a description of the parameter (optional)
+
+    Example:
+
+    .. literalinclude::
+       /../../tests/parameters/fixtures/hidden-param001.yaml
+       :language: yaml
+
+    """
+    base_param(parser, xml_parent, data, True,
+               'com.wangyin.parameter.WHideParameterDefinition')
+
+
+def random_string_param(registry, xml_parent, data):
+    """yaml: random-string
+    This parameter generates a random string and passes it to the
+    build, preventing Jenkins from combining queued builds.
+    Requires the Jenkins :jenkins-wiki:`Random String Parameter Plugin
+    <Random+String+Parameter+Plugin>`.
+
+    :arg str name: Name of the parameter
+    :arg str description: Description of the parameter (default '')
+    :arg str failed-validation-message: Failure message to display for invalid
+        input (default '')
+
+    Example:
+
+    .. literalinclude::
+       /../../tests/parameters/fixtures/random-string-param001.yaml
+       :language: yaml
+    """
+    pdef = XML.SubElement(xml_parent,
+                          'hudson.plugins.random__string__parameter.'
+                          'RandomStringParameterDefinition')
+    if 'name' not in data:
+        raise JenkinsJobsException('random-string must have a name parameter.')
+
+    mapping = [
+        ('name', 'name', None),
+        ('description', 'description', ''),
+        ('failed-validation-message', 'failedValidationMessage', ''),
+    ]
+    convert_mapping_to_xml(pdef, data, mapping, fail_required=True)
+
+
+def git_parameter_param(registry, xml_parent, data):
+    """yaml: git-parameter
+    This parameter allows to select a git tag, branch or revision number as
+    parameter in Parametrized builds.
+    Requires the Jenkins :jenkins-wiki:`Git Parameter Plugin
+    <Git+Parameter+Plugin>`.
+
+    :arg str name: Name of the parameter
+    :arg str description: Description of the parameter (default '')
+    :arg str type: The type of the list of parameters (default 'PT_TAG')
+
+        :Allowed Values: * **PT_TAG** list of all commit tags in repository -
+                        returns Tag Name
+                    * **PT_BRANCH** list of all branches in repository -
+                        returns Branch Name
+                    * **PT_BRANCH_TAG** list of all commit tags and all
+                        branches in repository - returns Tag Name or Branch
+                        Name
+                    * **PT_REVISION** list of all revision sha1 in repository
+                        followed by its author and date - returns Tag SHA1
+                    * **PT_PULL_REQUEST**
+
+    :arg str branch: Name of branch to look in. Used only if listing
+        revisions.  (default '')
+    :arg str branchFilter: Regex used to filter displayed branches. If blank,
+        the filter will default to ".*". Remote branches will be listed with
+        the remote name first. E.g., "origin/master"  (default '.*')
+    :arg str tagFilter: Regex used to filter displayed branches. If blank, the
+        filter will default to ".*". Remote branches will be listed with the
+        remote name first. E.g., "origin/master"  (default '*')
+    :arg str sortMode: Mode of sorting.  (default 'NONE')
+
+        :Allowed Values: * **NONE**
+                    * **DESCENDING**
+                    * **ASCENDING**
+                    * **ASCENDING_SMART**
+                    * **DESCENDING_SMART**
+
+    :arg str defaultValue: This value is returned when list is empty. (default
+        '')
+    :arg str selectedValue: Which value is selected, after loaded parameters.
+        If you choose 'default', but default value is not present on the list,
+        nothing is selected. (default 'NONE')
+
+        :Allowed Values: * **NONE**
+                    * **TOP**
+                    * **DEFAULT**
+
+    :arg str useRepository: If in the task is defined multiple repositories
+        parameter specifies which the repository is taken into account. If the
+        parameter is not defined, is taken first defined repository. The
+        parameter is a regular expression which is compared with a URL
+        repository. (default '')
+    :arg bool quickFilterEnabled: When this option is enabled will show a text
+        field. Parameter is filtered on the fly. (default false)
+
+    Minimal Example:
+
+    .. literalinclude::
+       /../../tests/parameters/fixtures/git-parameter-param-minimal.yaml
+       :language: yaml
+
+    Full Example:
+
+    .. literalinclude::
+       /../../tests/parameters/fixtures/git-parameter-param-full.yaml
+       :language: yaml
+    """
+    pdef = XML.SubElement(xml_parent,
+                          'net.uaznia.lukanus.hudson.plugins.gitparameter.'
+                          'GitParameterDefinition')
+
+    valid_types = [
+        'PT_TAG',
+        'PT_BRANCH',
+        'PT_BRANCH_TAG',
+        'PT_REVISION',
+        'PT_PULL_REQUEST',
+    ]
+
+    valid_sort_modes = [
+        'NONE',
+        'ASCENDING',
+        'ASCENDING_SMART',
+        'DESCENDING',
+        'DESCENDING_SMART',
+    ]
+
+    valid_selected_values = ['NONE', 'TOP', 'DEFAULT']
+
+    mapping = [
+        ('name', 'name', None),
+        ('description', 'description', ''),
+        ('type', 'type', 'PT_TAG', valid_types),
+        ('branch', 'branch', ''),
+        ('tagFilter', 'tagFilter', '*'),
+        ('branchFilter', 'branchFilter', '.*'),
+        ('sortMode', 'sortMode', 'NONE', valid_sort_modes),
+        ('defaultValue', 'defaultValue', ''),
+        ('selectedValue', 'selectedValue', 'NONE', valid_selected_values),
+        ('useRepository', 'useRepository', ''),
+        ('quickFilterEnabled', 'quickFilterEnabled', False),
+    ]
+    convert_mapping_to_xml(pdef, data, mapping, fail_required=True)
 
 
 class Parameters(jenkins_jobs.modules.base.Base):
@@ -766,7 +998,7 @@ class Parameters(jenkins_jobs.modules.base.Base):
     component_type = 'parameter'
     component_list_type = 'parameters'
 
-    def gen_xml(self, parser, xml_parent, data):
+    def gen_xml(self, xml_parent, data):
         properties = xml_parent.find('properties')
         if properties is None:
             properties = XML.SubElement(xml_parent, 'properties')
@@ -787,5 +1019,4 @@ class Parameters(jenkins_jobs.modules.base.Base):
             if pdefs is None:
                 pdefs = XML.SubElement(pdefp, 'parameterDefinitions')
             for param in parameters:
-                self.registry.dispatch('parameter',
-                                       parser, pdefs, param)
+                self.registry.dispatch('parameter', pdefs, param)
