@@ -23,18 +23,23 @@ import types
 
 from jenkins_jobs.errors import JenkinsJobsException
 from jenkins_jobs.formatter import deep_format
+from jenkins_jobs.local_yaml import Jinja2Loader
+
+__all__ = [
+    "ModuleRegistry"
+]
 
 logger = logging.getLogger(__name__)
 
 
 class ModuleRegistry(object):
-    entry_points_cache = {}
+    _entry_points_cache = {}
 
-    def __init__(self, config, plugins_list=None):
+    def __init__(self, jjb_config, plugins_list=None):
         self.modules = []
         self.modules_by_component_type = {}
         self.handlers = {}
-        self.global_config = config
+        self.jjb_config = jjb_config
         self.masked_warned = {}
 
         if plugins_list is None:
@@ -60,7 +65,7 @@ class ModuleRegistry(object):
             mapped to its plugin info dictionary.
             """
             version = plugin_info.get('version', '0')
-            plugin_info['version'] = re.sub(r'(.*)-(?:SNAPSHOT|BETA)',
+            plugin_info['version'] = re.sub(r'(.*)-(?:SNAPSHOT|BETA).*',
                                             r'\g<1>.preview', version)
 
             aliases = []
@@ -118,8 +123,14 @@ class ModuleRegistry(object):
     def getHandler(self, category, name):
         return self.handlers[category][name]
 
-    def dispatch(self, component_type,
-                 parser, xml_parent,
+    @property
+    def parser_data(self):
+        return self.__parser_data
+
+    def set_parser_data(self, parser_data):
+        self.__parser_data = parser_data
+
+    def dispatch(self, component_type, xml_parent,
                  component, template_data={}):
         """This is a method that you can call from your implementation of
         Base.gen_xml or component.  It allows modules to define a type
@@ -150,25 +161,26 @@ class ModuleRegistry(object):
         if isinstance(component, dict):
             # The component is a singleton dictionary of name: dict(args)
             name, component_data = next(iter(component.items()))
-            if template_data:
+            if template_data or isinstance(component_data, Jinja2Loader):
                 # Template data contains values that should be interpolated
-                # into the component definition
-                allow_empty_variables = self.global_config \
-                    and self.global_config.has_section('job_builder') \
-                    and self.global_config.has_option(
-                        'job_builder', 'allow_empty_variables') \
-                    and self.global_config.getboolean(
-                        'job_builder', 'allow_empty_variables')
-
-                component_data = deep_format(
-                    component_data, template_data, allow_empty_variables)
+                # into the component definition.  To handle Jinja2 templates
+                # that don't contain any variables, we also deep format those.
+                try:
+                    component_data = deep_format(
+                        component_data, template_data,
+                        self.jjb_config.yamlparser['allow_empty_variables'])
+                except Exception:
+                    logging.error(
+                        "Failure formatting component ('%s') data '%s'",
+                        name, component_data)
+                    raise
         else:
             # The component is a simple string name, eg "run-tests"
             name = component
             component_data = {}
 
         # Look for a component function defined in an entry point
-        eps = ModuleRegistry.entry_points_cache.get(component_list_type)
+        eps = self._entry_points_cache.get(component_list_type)
         if eps is None:
             module_eps = []
             # auto build entry points by inferring from base component_types
@@ -220,29 +232,28 @@ class ModuleRegistry(object):
                 eps[module_ep.name] = module_ep
 
             # cache both sets of entry points
-            ModuleRegistry.entry_points_cache[component_list_type] = eps
+            self._entry_points_cache[component_list_type] = eps
             logger.debug("Cached entry point group %s = %s",
                          component_list_type, eps)
 
         # check for macro first
-        component = parser.data.get(component_type, {}).get(name)
+        component = self.parser_data.get(component_type, {}).get(name)
         if component:
             if name in eps and name not in self.masked_warned:
-                # Warn only once for each macro
                 self.masked_warned[name] = True
-                logger.warn("You have a macro ('%s') defined for '%s' "
-                            "component type that is masking an inbuilt "
-                            "definition" % (name, component_type))
+                logger.warning(
+                    "You have a macro ('%s') defined for '%s' "
+                    "component type that is masking an inbuilt "
+                    "definition" % (name, component_type))
 
             for b in component[component_list_type]:
                 # Pass component_data in as template data to this function
                 # so that if the macro is invoked with arguments,
                 # the arguments are interpolated into the real defn.
-                self.dispatch(component_type,
-                              parser, xml_parent, b, component_data)
+                self.dispatch(component_type, xml_parent, b, component_data)
         elif name in eps:
             func = eps[name].load()
-            func(parser, xml_parent, component_data)
+            func(self, xml_parent, component_data)
         else:
             raise JenkinsJobsException("Unknown entry point or macro '{0}' "
                                        "for component type: '{1}'.".
